#!/usr/local/bin/python2.1
#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# 

import sys
import getopt
import struct
import time
import ipmichassis
import ipmicommon
import ipmievent
import ipmisdr
import ipmisensor
import ipmitransport

x = None

def usage():
    print "usage: %s -a|-k|-m  -l|-p S|-r N|-s" % sys.argv[0]
    print "Mode:"
    print "  -a: IP Address for IPMI LAN session"
    print "  -k: KCS System Interface"
    print "  -m: SMIC System Interface"
    print ""
    print "Command:"
    print "  -l: Display System Event Log"
    print "  -p: Power control:"
    print "      'off' = Power off"
    print "      'on' = Power on"
    print "      'cycle' = Power cycle (off for 2s, then on)"
    print "      'reset' = Reset"
    print "      'nmi' = NMI"
    print "  -r: Get Sensor Reading for sensor device N"
    print "  -s: Dump SDR Repository"

def toomanyopts():
    print "Must choose either -l, -p, -r, or -s option"
    usage()
    sys.exit(1)

def toomanymodes():
    print "Must choose either -a, -k, or -m option"
    usage()
    sys.exit(1)
    
def pbytes(ret):
    for y in ret:
        print "0x%x ," % ord(y)

def utos(val, bits):
    if val & (2**(bits-1)):
        return -((~val & 2**(bits-1)-1)+1)
    else:
        return val

def dopower(arg):
    # power control
    ipmichassis.control(x, arg)
    
##    ret = x.send(0x20, 0x0, 0x81, 0x0, 41, 0x0, 0x2, struct.pack('<B', int(arg)))
##    pbytes(ret)
        
def doseldump():
    
    # event log dump
    # To dump the entire log, we must first throw down a reservation
    # to track if it changes while we're reading it.
    # Reservation is 0xA/0x42
    sels = []
    ret = x.send(0x20, 0x0, 0x81, 0x0, 0x0, 0xA, 0x42, '')
    #bytes(ret)
    if ord(ret[0]) != 0x0:
        print "Couldn't reserve SEL, being modified (0x%x)" % ord(ret[0])
        sys.exit(2)
    resvid = (ord(ret[2])<<8) + ord(ret[1])

    seloff = 0
    while seloff != 0xffff:
        #print "Getting record ID 0x%x" % seloff
##        ret = x.send(0x20, 0x0, 0x81, 0x0, 0x0, 0xA, 0x43, struct.pack('<6B', resvid & 0x00ff, (resvid & 0xff00) >> 8, seloff & 0x00ff, (seloff & 0xff00) >> 8, 0x0, 0xff))
##        #pbytes(ret)
##        if ord(ret[0]) == 0x81 or ord(ret[0]) == 0xC5:
##            print "Reservation cancelled, SEL modification (0x%x)" % ord(ret[0])
##            break;
        ret = ipmitransport.load_chunked_record(x, 0x20, 0x0, 0x81, 0x0, 0x0, 0xA, 0x43, resvid, seloff, 16)
        seloff = (ord(ret[2]) << 8) + ord(ret[1])
        sels.append(ret[3:])

    #print "Retrieved SELs:"
    #print sels

    # Try to parse out.
    for y in sels:
        # 1,2: Record ID, LSB first
        # 3: Record Type, 0x2 = system, 0xc0-0xdf = ts, 0xe0-0xff no ts
        # 4-7: Timestamp, LSB first
        # 8,9: Generator ID (RqSA & LUN or Software ID)
        #      8: Addr or SWID
        #      9: Channel, bit 1:0 = LUN
        # 10: Event Message Revision (==0x4)
        # 11: Sensor Type
        # 12: Sensor Number
        # 13: Event Direction: 7: assert/deassert; 6:0: Type code
        # 14,15,16: Event Data 1,2,3



        ridlo, ridhi, rectype, tstamp, gidlo, gidhi, emr, stype, snum, sdir, data1, data2, data3 = struct.unpack('<3BL9B', y )

        print "%0.4d 0x%x %s 0x%0.2x 0x%0.2x 0x%0.2x 0x%x 0x%x %c 0x%x 0x%x 0x%x 0x%x" % ( ((ridhi << 8) + ridlo), rectype, time.ctime(tstamp), gidhi, gidlo & 0xf0, emr, stype, snum, sdir & 0x80 >> 7, sdir, data1, data2, data3)


        # Important bits for interpretation:

        # Sensor type (36-3)
        # Event Type (section 36)
        # Event Data (dependent on type)

        # The Event Type dictates how the Event Data bytes
        # are parsed.
        # eoff is the Event Offset, selecting the specific transition
        # that caused the sensor to trigger.

        # IPMI 0.9 (emr 0x2) defined etypes 0xe6 and 0xe7 for
        # sensor-specific event codes. Translate into IPMI 1.5
        # sensor-specific codes. 0xe6 is a deassert and 0xe7 is
        # an assert. (these were broken out into the direction bit)

        if emr == 0x2:
            if sdir == 0xe6 or sdir == 0xe7:
                etype = 0x6f
        else:
            etype = sdir & 0x7f

        etypestr = ""
        especstr = ""
        eoff = data1 & 0xf

        (etypestr, especstr) = ipmievent.decode_event(etype, eoff, stype, data1, data2, data3)

        print "%s sel[%d]: %s - %s" % (time.ctime(tstamp), ((ridhi << 8) + ridlo), etypestr, especstr)

        # END EVENT LOG DUMP FUNCTION

### Sensor Reading ####################################################
def dosensread(arg):

    sensor = int(arg, 0)
    sdrs = ipmisdr.download_sdr(x)

    rec = sdrs[(0x1, 0x20, 0x0, sensor)]

    sensdata = ipmisensor.get_sensor_reading(x, 0x20, 0x0, sensor)
    out = ipmisensor.convert_raw(sensdata[0], rec)

    print "%s = %s" % (rec['id-string'], out)

    # previous code below

##    ret = x.send(0x20, 0x0, 0x81, 0x0, 1, 0x4, 0x2D, struct.pack('<B', int(arg)))
##    # only thresh-type sensors for now
##    ccode, reading, evtflag, thresh = struct.unpack('<4B', ret[:4])

##    if ccode != 0x0:
##        print "Get Sensor Reading error (%d)" % ccode
##        return -1

##    print "Get Sensor Reading:"
##    print "Sensor Reading          : %d" % reading
##    print "Event Flags             : 0x%x" % evtflag
##    if not evtflag & 0x80:
##        print "                        : Events Disabled for this sensor"
##    if not evtflag & 0x40:
##        print "                        : Sensor Scanning disabled"
##    if evtflag & 0x20:
##        print "                        : Initial Update in progress"
##    print "Threshold Status        : 0x%x" % thresh
##    if thresh & 0x20:
##        print "                        : >= upper non-recoverable thresh"
##    if thresh & 0x10:
##        print "                        : >= upper critical thresh"
##    if thresh & 0x8:
##        print "                        : >= upper non-critical thresh"
##    if thresh & 0x4:
##        print "                        : <= lower non-recoverable thresh"
##    if thresh & 0x2:
##        print "                        : <= lower critical thresh"
##    if thresh & 0x1:
##        print "                        : <= lower non-critical thresh"

        
    
### SDR Dump #####################################################

def dosdrdump():
    #if function == "-s":
    # Pull out the SDR Repo Info

    ret = x.send(0x20, 0x0, 0x81, 0x0, 1, 0xa, 0x20, '')
    #pbytes(ret)

    retcode, sdrver, reclo, rechi, freelo, freehi, addts, erasets, opsup = struct.unpack('<BBBBBBLLB', ret)

    print "SDR Repository Information:"
    print "SDR Version          : 0x%x" % sdrver
    print "Record Count         : %d" % ((rechi << 8) + reclo)
    print "Bytes Free           : %d" % ((freehi << 8) + freelo)
    print "Last Record Added    : %s" % time.ctime(addts)
    print "Last Record Modified : %s" % time.ctime(erasets)
    print "Operational Support  : 0x%x" % opsup
    if opsup & 0x1:
        print "                     : Get SDR Allcation Info cmd supported"
    if opsup & 0x2:
        print "                     : Reserve SDR Repo cmd supported"
    if opsup & 0x4:
        print "                     : Partial Add SDR cmd supported"
    if opsup & 0x8:
        print "                     : Delete SDR cmd supported"
    if opsup & 0x20 and opsup & 0x40 == 0:
        print "                     : Non-modal SDR Update supported"
    if opsup & 0x20 == 0 and opsup & 0x40:
        print "                     : Modal SDR Update supported"
    if opsup & 0x20 and opsup & 0x40:
        print "                     : Both Modal and Non-modal update supported"
    if opsup & 0x80:
        print "                     : Overflow Flag Set"


    # Grab the repo
    sdrs = ipmisdr.download_sdr(x)

    for k in sdrs.keys():
        s = sdrs[k]
        print "------------------------------------------------------------"
        if s['record-type'] == 0x1:
            print "ID String                        : %s" % s['id-string']
            print "Sensor Owner ID/Lun/Number       : 0x%x/0x%x/0x%x" % (s['sensor-owner-id'], s['sensor-owner-lun'], s['sensor-number'])
            print "Entity ID/Instance               : 0x%x/0x%x" % (s['entity-id'], s['entity-instance'])
            
            print "                                 : %s" % s['entity-id-name']
            scap = s['sensor-capabilities']
            scaphyst = s['sensor-capability-hysteresis']
            scapthresh = s['sensor-capability-thresh']
            scapevent = s['sensor-capability-events']
            print "Sensor Capabilities              : 0x%x" % scap
            if scap & 0x80:
                print "                                 : Ignore if Entity not present"
            if scap & 0x40:
                print "                                 : Auto re-arming"
            if scaphyst == 0:
                print "                                 : Hysteresis unsupported"
            if scaphyst == 1:
                print "                                 : Hysteresis readable"
            if scaphyst == 2:
                print "                                 : Hysteresis read/write"
            if scaphyst == 3:
                print "                                 : Hysteresis built-in"
            if scapthresh == 0:
                print "                                 : No thresholds"
            if scapthresh == 1:
                print "                                 : Thresholds readable"
            if scapthresh == 2:
                print "                                 : Thresholds read/write"
            if scapthresh == 3:
                print "                                 : Thresholds built-in"
            if scapevent == 0:
                print "                                 : Per-threshold event control"
            if scapevent == 1:
                print "                                 : Entire sensor event control only"
            if scapevent == 2:
                print "                                 : Global disable only"
            if scapevent == 3:
                print "                                 : Sensor does not send events"
            print "Sensor Type                      : 0x%x" % s['sensor-type']
            if s.has_key('sensor-type-name'):
                print "                                 : %s" % s['sensor-type-name']
            print "Event Type                       : 0x%x" % s['event-type']
            if s.has_key('event-type-name'):
                print "                                 : %s" % s['event-type-name']
            print "Assertion Event/Thresh Read Mask : 0x%x" % s['assert-event-mask']
            print "Deassert Event/Upper Thresh Mask : 0x%x" % s['deassert-event-mask']
            print "Disc Read/Read/Write Thresh Mask : 0x%x" % s['discrete-reading-mask']
            print "Sensor Units                     : 0x%x" % s['sensor-units-1']
            adf = s['analog-data-format']
            rate = s['rate-unit']
            if adf == 0:
                print "                                 : Unsigned"
            if adf == 1:
                print "                                 : 1's Comp Signed"
            if adf == 2:
                print "                                 : 2's Comp Signed"
            if adf == 3:
                print "                                 : Non-numeric reading"
            if rate == 0:
                print "                                 : No rate unit"
            if rate == 1:
                print "                                 : Per microsend"
            if rate == 2:
                print "                                 : Per milisecond"
            if rate == 3:
                print "                                 : Per second"
            if rate == 4:
                print "                                 : Per minute"
            if rate == 5:
                print "                                 : Per hour"
            if rate == 6:
                print "                                 : Per day"
            mut = s['modifier-unit-type']
            if mut == 0:
                print "                                 : No rate unit"
            if mut == 1:
                print "                                 : Basic Divided by Modifier Unit"
            if mut == 2:
                print "                                 : Basic Multipled by Modifier Unit"
            if s['unit-percentage']:
                print "                                 : Percentage"
            print "Base Unit                        : %s" % s['base-unit']
            print "Modifier Unit                    : %s" % s['modifier-unit']
            print "Linearization                    : %d" % s['linearization']
            print "M coefficient                    : %d" % s['m']
            print "B scalar                         : %d" % s['b']
            print "Tolerance                        : %d" % s['tolerance']
            print "Accuracy                         : %d" % s['accuracy']
            print "R/B Exponent                     : %d/%d" % (s['r-exponent'], s['b-exponent'])
            achar = s['analog-characteristics']
            print "Analog characteristics           : 0x%x" % achar
            if achar & 0x4:
                print "                                 : Normal Minumum specified"
            if achar & 0x2:
                print "                                 : Normal Maximum specified"
            if achar & 0x1:
                print "                                 : Nominal Reading specified"
            print "Nominal Reading                  : %s" % ipmisensor.convert_raw(s['nominal-reading'], s)
            print "Normal Maxiumum                  : %s" % ipmisensor.convert_raw(s['normal-maximum'], s)
            print "Normal Minumum                   : %s" % ipmisensor.convert_raw(s['normal-minimum'], s)
            print "Sensor Maximum Reading           : %s" % ipmisensor.convert_raw(s['sensor-maximum-reading'], s)
            print "Sensor Minumum Reading           : %s" % ipmisensor.convert_raw(s['sensor-minimum-reading'], s)
            print "Upper Non-recoverable Threshold  : %s" % ipmisensor.convert_raw(s['upper-non-recoverable-thresh'], s)
            print "Upper Critical Threshold         : %s" % ipmisensor.convert_raw(s['upper-critical-thresh'], s)
            print "Upper Non-critical Threshold     : %s" % ipmisensor.convert_raw(s['upper-non-critical-thresh'], s)
            print "Lower Non-recoverable Threshold  : %s" % ipmisensor.convert_raw(s['lower-non-recoverable-thresh'], s)
            print "Lower Critical Threshold         : %s" % ipmisensor.convert_raw(s['lower-critical-thresh'], s)
            print "Lower Non-critical Threshold     : %s" % ipmisensor.convert_raw(s['lower-non-critical-thresh'], s)
            print "Postive-going Hysteresis         : %d" % s['positive-going-hysteresis']
            print "Negative-going Hysteresis        : %d" % s['negative-going-hysteresis']
            print "ID Type/Length (packed)          : 0x%x" % s['id-type']

            if s.has_key('raw-b'):
                print "B: 0x%x" % s['raw-b']
            if s.has_key('raw-b-accuracy'):
                print "BACC: 0x%x" % s['raw-b-accuracy']


    # this is all old code from below here

    
    # okay, now dump the repo
    # we have to play the same 'reserve the repo' game as with
    # the event log

##    if opsup & 0x2:
##        # print ">> Getting reservation"
##        ret = x.send(0x20, 0x0, 0x81, 0x0, 0x0, 0xA, 0x22, '')
##        #bytes(ret)
##        if ord(ret[0]) != 0x0:
##            print "Couldn't reserve SDR, being modified (0x%x)" % ord(ret[0])
##            sys.exit(2)
##        resvid = (ord(ret[2])<<8) + ord(ret[1])
##    else:
##        resvid = 0

    # Extract the actual records,incrementally
    # Since SDRs are big, we need to read them chunked. The usual chunk
    # size limit is in the 30 byte range.
    #
    # XXX Since we only want the first 6 bytes or so, limit our
    # consumption to that until we can do proper chunking.
    ##sdrs = []
##    seloff = 0
##    while seloff != 0xffff:
##        print "Getting record ID 0x%x" % seloff
##        # XXX
##        ret = x.send(0x20, 0x0, 0x81, 0x0, 0x0, 0xA, 0x23, struct.pack('<6B', resvid & 0x00ff, (resvid & 0xff00) >> 8, seloff & 0x00ff, (seloff & 0xff00) >> 8, 0x0, 0x5))
##        # ret = x.send(0x20, 0x0, 0x81, 0x0, 0x0, 0xA, 0x23, struct.pack('<6B', resvid & 0x00ff, (resvid & 0xff00) >> 8, seloff & 0x00ff, (seloff & 0xff00) >> 8, 0x0, 10))
##        pbytes(ret)
##        if ord(ret[0]) == 0x81 or ord(ret[0]) == 0xC5:
##            print "Reservation cancelled, SDR modification (0x%x)" % ord(ret[0])
##            break
##        elif ord(ret[0]) != 0x0:
##            print "Error occured getting SDR (0x%x)" % ord(ret[0])
##            break

##        # Parse out header.
##        # len is the number of bytes yet to grab
##        ridlo, ridhi, sdrver, rtype, rlen = struct.unpack('<BBBBB', ret[3:8])
##        print "#%d: Type 0x%x, len %d" % ((ridhi << 8)+ridlo, rtype, rlen)

##        chunkoff = 5
##        chunklen = 30
##        # Grab the rest of the bytes.
##        while rlen > 0:
##            # Grab 30 byte chunks until len is depleted
##            # Looks like the 'bytes to read too big' thing isn't portable.
##            if rlen < chunklen:
##                chunklen = rlen

##            # need to rip off completion code & header info again
##            print ">> Getting chunk, offset = %d, len = %d" % (chunkoff, chunklen)
##            cbody = x.send(0x20, 0x0, 0x81, 0x0, 0x0, 0xA, 0x23, struct.pack('<6B', resvid & 0x00ff, (resvid & 0xff00) >> 8, seloff & 0x00ff, (seloff & 0xff00) >> 8, chunkoff, chunklen))
##            if ord(cbody[0]) != 0x0:
##                if ord(cbody[0]) == 0xc9 and chunklen != 16:
##                    # "Parameter out of range"
##                    # on the HPs, they can only handle a max chunksize
##                    # of 16 bytes, while the Intels can do better
##                    # than 30. Let's try resetting the chunksize
##                    # and giving it another spin.
##                    print "**** Trying chunklen = 16"
##                    chunklen = 16
##                    continue
##                else:
##                    print "******* Error getting chunk at len %d (0x%x)" % (chunkoff, ord(cbody[0]))
##                    #pbytes(cbody)
##                    break
##            print ">> Got %d bytes in last chunk" % len(cbody[3:])
##            ret = ret + cbody[3:]
##            rlen = rlen - chunklen
##            chunkoff = chunkoff + chunklen                           

        ##ret = ipmitransport.load_chunked_record(x, 0x20, 0x0, 0x81, 0x0, 0x0, 0xA, 0x23, resvid, seloff, 0)

        # Inject body sans response data into list.
        ##sdrs.append(ret[3:])

##        # Update offset.
##        seloff = (ord(ret[2]) << 8) + ord(ret[1])
        

    # I wanted to build the SDRs as a dictionary, but it's probably
    # more effecient to slurp into a list then rebuild the
    # dictionary after we have all the records since the
    # data format changes on the type.

##    print "Number of SDR Records returned: %d" % len(sdrs)

##    for s in sdrs:
##        # record header
##        ridlo, ridhi, sdrver, rtype, rlen = struct.unpack('<BBBBB', s[:5])
##        print "#%d: Type 0x%x, version 0x%x, len %d" % ((ridhi << 8)+ridlo, rtype, sdrver, rlen)

        # print "decode_sdr_record sez: %s" % ipmisdr.decode_sdr_record(s)

##        if rtype == 0x1:
##            # Full sensor record
##            # leaving off the id string
##            #print "Trying len %d" % len(s[5:48])
##            #
##            # Some of these values are composited from other bytes.
##            # Some are in 2s complement.
##            #
##            # M, B, Rexp, Bexp
##            # Sensor max/mins may be signed as per Units flags

##            try:
##                sownid, solun, snum, entid, entinst, sinit, scap, stype, etype, amasklo, amaskhi, dmasklo, dmaskhi, dimasklo, dimaskhi, sunit1, sunit2, sunit3, lin, m, mtol, b, bacc, acc, rbexp, achar, nread, nmax, nmin, smax, smin, unonrecov, ucrit, unoncrit, lnonrecov, lcrit, lnoncrit, physt, nhyst, rev1, rev2, oem, idtype = struct.unpack('<BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB', s[5:48])
##                idstr = s[48:]
##            except:
##                print "struct bombed, len = %d" % len(s[5:48])
##                pbytes(s[5:48])
##                raise

##            # Print some info.
##            print "Sensor Owner ID/Lun/Number       : 0x%x/0x%x/0x%x" % (sownid, solun, snum)
##            print "Entity ID/Instance               : 0x%x/0x%x" % (entid, entinst)
##            if entid < len(ipmievent.entitytypelist):
##                print "                                 : %s" % ipmievent.entitytypelist[entid]
##            print "Sensor Capabilities              : 0x%x" % (scap)
##            if scap & 0x80:
##                print "                                 : Ignore if Entity not present"
##            if scap & 0x40:
##                print "                                 : Auto re-arming"
##            if scap & 0x20 == 0 and scap & 0x10 == 0:
##                print "                                 : Hysteresis unsupported"
##            if scap & 0x20 == 0 and scap & 0x10:
##                print "                                 : Hysteresis readable"
##            if scap & 0x20 and scap & 0x10 == 0:
##                print "                                 : Hysteresis read/write"
##            if scap & 0x20 and scap & 0x10:
##                print "                                 : Hysteresis built-in"
##            if scap & 0x8 == 0 and scap & 0x4 == 0:
##                print "                                 : No thresholds"
##            if scap & 0x8 == 0 and scap & 0x4:
##                print "                                 : Thresholds readable"
##            if scap & 0x8 and scap & 0x4 == 0:
##                print "                                 : Thresholds read/write"
##            if scap & 0x8 and scap & 0x4:
##                print "                                 : Thresholds built-in"
##            if scap & 0x2 == 0 and scap & 0x1 == 0:
##                print "                                 : Per-threshold event control"
##            if scap & 0x2 == 0 and scap & 0x1:
##                print "                                 : Entire sensor event control only"
##            if scap & 0x2 and scap & 0x1 == 0:
##                print "                                 : Global disable only"
##            if scap & 0x2 and scap & 0x1:
##                print "                                 : Sensor does not send events"
##            print "Sensor Type                      : 0x%x" % stype
##            if stype < len(ipmievent.sensortypelist):
##                print "                                 : %s" % ipmievent.sensortypelist[stype]
##            print "Event Type                       : 0x%x" % etype
##            if etype < len(ipmievent.eventtypelist):
##                print "                                 : %s" % ipmievent.eventtypelist[etype]
##            print "Assertion Event/Thresh Read Mask : 0x%0.4x%0.4x" % (amaskhi, amasklo)
##            print "Deassert Event/Upper Thresh Mask : 0x%0.4x%0.4x" % (dmaskhi, dmasklo)
##            print "Disc Read/Read/Write Thresh Mask : 0x%0.4x%0.4x" % (dimaskhi, dimasklo)
##            print "Sensor Units                     : 0x%x" % (sunit1)
##            if sunit1 & 0x80 == 0 and sunit1 & 0x40 == 0:
##                print "                                 : Unsigned"
##            if sunit1 & 0x80 == 0 and sunit1 & 0x40:
##                print "                                 : 1's Comp Signed"
##            if sunit1 & 0x80 and sunit1 & 0x40 == 0:
##                print "                                 : 2's Comp Signed"
##            if sunit1 & 0x80 and sunit1 & 0x40:
##                print "                                 : Non-numeric reading"
##            if not sunit1 & 0x20 and not sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : No rate unit"
##            if not sunit1 & 0x20 and not sunit1 & 0x10 and sunit1 & 0x8:
##                print "                                 : Per microsend"
##            if not sunit1 & 0x20 and  sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : Per milisecond"
##            if not sunit1 & 0x20 and  sunit1 & 0x10 and  sunit1 & 0x8:
##                print "                                 : Per second"
##            if sunit1 & 0x20 and not sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : Per minute"
##            if sunit1 & 0x20 and not sunit1 & 0x10 and sunit1 & 0x8:
##                print "                                 : Per hour"
##            if sunit1 & 0x20 and sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : Per day"
##            if sunit1 & 0x20 and not sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : No rate unit"
##            if not sunit1 & 0x4 and sunit1 & 0x2:
##                print "                                 : Basic Divided by Modifier Unit"
##            if sunit1 & 0x4 and not sunit1 & 0x2:
##                print "                                 : Basic Multipled by Modifier Unit"
##            if sunit1 & 0x1:
##                print "                                 : Percentage"
##            print "Base Unit                        : 0x%x" % sunit2
##            if sunit2 < len(ipmievent.unittypelist):
##                print "                                 : %s" % ipmievent.unittypelist[sunit2]
##            print "Modifier Unit                    : 0x%x" % sunit3
##            if sunit3 < len(ipmievent.unittypelist):
##                print "                                 : %s" % ipmievent.unittypelist[sunit3]
##            print "Linearization                    : %d" % lin
##            m = utos(((mtol & 0xc0) << 8) + m, 10)
##            # print "M coefficient                    : %d" % (((mtol & 0xc0) << 8) + m)
##            print "M coefficient                    : %d" % m
##            b = utos(((bacc & 0xc0) << 8) + b, 10)
##            print "B scalar                         : %d" % (((bacc & 0xc0) << 8) + b)
##            print "Tolerance                        : %d" % (mtol & 0x3f)
##            print "Accuracy                         : %d" % (((acc & 0xf0) << 2) + (bacc & 0x3f))
##            # check bit math
##            print "Accuracy Exponent                : %d" % ((acc & 0xc) >> 3)
##            rexp = utos((rbexp & 0xf0) >> 4, 4)
##            bexp = utos(rbexp & 0xf, 4)
##            # print "R/B Exponent                     : %d/%d" % ((rbexp & 0xf0) >> 4, rbexp & 0xf )
##            print "R/B Exponent                     : (0x%x) %d/%d" % (rbexp, rexp, bexp)
##            print "Analog characteristics           : 0x%x" % achar
##            if achar & 0x4:
##                print "                                 : Normal Minumum specified"
##            if achar & 0x2:
##                print "                                 : Normal Maximum specified"
##            if achar & 0x1:
##                print "                                 : Nominal Reading specified"
##            print "Nominal Reading                  : %d" % nread
##            print "Normal Maxiumum                  : %d" % nmax
##            print "Normal Minumum                   : %d" % nmin
##            print "Sensor Maximum Reading           : %d" % smax
##            print "Sensor Minumum Reading           : %d" % smin
##            print "Upper Non-recoverable Threshold  : %d" % unonrecov
##            print "Upper Critical Threshold         : %d" % ucrit
##            print "Upper Non-critical Threshold     : %d" % unoncrit
##            print "Lower Non-recoverable Threshold  : %d" % lnonrecov
##            print "Lower Critical Threshold         : %d" % lcrit
##            print "Lower Non-critical Threshold     : %d" % lnoncrit
##            print "Postive-going Hysteresis         : %d" % physt
##            print "Negative-going Hysteresis        : %d" % nhyst
##            print "ID Type/Length (packed)          : 0x%x" % idtype
##            print "ID String                        : %s" % idstr


##        if rtype == 0x2:
##            try:
##                sownid, solun, snum, entid, entinst, sinit, scap, stype, etype, amasklo, amaskhi, dmasklo, dmaskhi, dimasklo, dimaskhi,sunit1, sunit2, sunit3, sshare1, sshare2, physt, nhyst, rev1, rev2, rev3, oem, idtype = struct.unpack('<27B', s[5:32])
##                idstr = s[32:]
##            except:
##                print "struct bombed, len = %d" % len(s[5:32])
##                pbytes(s[5:])
##                raise

##            print "Sensor Owner ID/Lun/Number       : 0x%x/0x%x/0x%x" % (sownid, solun, snum)
##            print "Entity ID/Instance               : 0x%x/0x%x" % (entid, entinst)
##            if entid < len(ipmievent.entitytypelist):
##                print "                                 : %s" % ipmievent.entitytypelist[entid]
##            print "Sensor Capabilities              : 0x%x" % (scap)
##            if scap & 0x80:
##                print "                                 : Ignore if Entity not present"
##            if scap & 0x40:
##                print "                                 : Auto re-arming"
##            if scap & 0x20 == 0 and scap & 0x10 == 0:
##                print "                                 : Hysteresis unsupported"
##            if scap & 0x20 == 0 and scap & 0x10:
##                print "                                 : Hysteresis readable"
##            if scap & 0x20 and scap & 0x10 == 0:
##                print "                                 : Hysteresis read/write"
##            if scap & 0x20 and scap & 0x10:
##                print "                                 : Hysteresis built-in"
##            if scap & 0x8 == 0 and scap & 0x4 == 0:
##                print "                                 : No thresholds"
##            if scap & 0x8 == 0 and scap & 0x4:
##                print "                                 : Thresholds readable"
##            if scap & 0x8 and scap & 0x4 == 0:
##                print "                                 : Thresholds read/write"
##            if scap & 0x8 and scap & 0x4:
##                print "                                 : Thresholds built-in"
##            if scap & 0x2 == 0 and scap & 0x1 == 0:
##                print "                                 : Per-threshold event control"
##            if scap & 0x2 == 0 and scap & 0x1:
##                print "                                 : Entire sensor event control only"
##            if scap & 0x2 and scap & 0x1 == 0:
##                print "                                 : Global disable only"
##            if scap & 0x2 and scap & 0x1:
##                print "                                 : Sensor does not send events"
##            print "Sensor Type                      : 0x%x" % stype
##            if stype < len(ipmievent.sensortypelist):
##                print "                                 : %s" % ipmievent.sensortypelist[stype]
##            print "Event Type                       : 0x%x" % etype
##            if etype < len(ipmievent.eventtypelist):
##                print "                                 : %s" % ipmievent.eventtypelist[etype]
##            print "Assertion Event/Thresh Read Mask : 0x%0.4x%0.4x" % (amaskhi, amasklo)
##            print "Deassert Event/Upper Thresh Mask : 0x%0.4x%0.4x" % (dmaskhi, dmasklo)
##            print "Disc Read/Read/Write Thresh Mask : 0x%0.4x%0.4x" % (dimaskhi, dimasklo)
##            print "Sensor Units                     : 0x%x" % (sunit1)
##            if sunit1 & 0x80 == 0 and sunit1 & 0x40 == 0:
##                print "                                 : Unsigned"
##            if sunit1 & 0x80 == 0 and sunit1 & 0x40:
##                print "                                 : 1's Comp Signed"
##            if sunit1 & 0x80 and sunit1 & 0x40 == 0:
##                print "                                 : 2's Comp Signed"
##            if sunit1 & 0x80 and sunit1 & 0x40:
##                print "                                 : Non-numeric reading"
##            if not sunit1 & 0x20 and not sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : No rate unit"
##            if not sunit1 & 0x20 and not sunit1 & 0x10 and sunit1 & 0x8:
##                print "                                 : Per microsend"
##            if not sunit1 & 0x20 and  sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : Per milisecond"
##            if not sunit1 & 0x20 and  sunit1 & 0x10 and  sunit1 & 0x8:
##                print "                                 : Per second"
##            if sunit1 & 0x20 and not sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : Per minute"
##            if sunit1 & 0x20 and not sunit1 & 0x10 and sunit1 & 0x8:
##                print "                                 : Per hour"
##            if sunit1 & 0x20 and sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : Per day"
##            if sunit1 & 0x20 and not sunit1 & 0x10 and not sunit1 & 0x8:
##                print "                                 : No rate unit"
##            if not sunit1 & 0x4 and sunit1 & 0x2:
##                print "                                 : Basic Divided by Modifier Unit"
##            if sunit1 & 0x4 and not sunit1 & 0x2:
##                print "                                 : Basic Multipled by Modifier Unit"
##            if sunit1 & 0x1:
##                print "                                 : Percentage"
##            print "Base Unit                        : 0x%x" % sunit2
##            if sunit2 < len(ipmievent.unittypelist):
##                print "                                 : %s" % ipmievent.unittypelist[sunit2]
##            print "Modifier Unit                    : 0x%x" % sunit3
##            if sunit3 < len(ipmievent.unittypelist):
##                print "                                 : %s" % ipmievent.unittypelist[sunit3]
##            print "Sensor Record Sharing 1          : 0x%x" % sshare1
##            if sshare1 & 0x10:
##                print "                                 : Append alpha chars to ID string"
##            print "Record Share References          : %d" % (sshare1 & 0xf)
##            print "Sensor Record Sharing 2          : 0x%x" % sshare2
##            if sshare2 & 0x80:
##                print "                                 : Entity increments for each share"
##            print "ID String Modification Offset    : 0x%x" % (sshare2 & 0x7f)
##            print "Postive-going Hysteresis         : %d" % physt
##            print "Negative-going Hysteresis        : %d" % nhyst
##            print "ID Type/Length (packed)          : 0x%x" % idtype
##            print "ID String                        : %s" % idstr


##        # FRU
##        if rtype == 0x11:
##            try:
##                daa, fruid, accessinfo, chan, rev1, devtype, devtmod, fruentid, fruentinst, oem, idtype = struct.unpack('<11B', s[5:16])
##                idstr = s[16:]
##            except:
##                print "struct bombed, len = %d" % len(s[5:16])
##                pbytes(s[5:])
##                raise

##            logical = 0

##            print "Device Access Address            : 0x%x" % daa
##            print "Device Access Channel            : 0x%x" % ((chan & 0xf0) >> 4)
##            print "FRU Access Info                  : 0x%x" % accessinfo
##            if accessinfo & 0x80:
##                print "                                 : Logical Device"
##                logical = 1
##            else:
##                print "                                 : Physical Device"

##            if logical:
##                print "FRU Device ID                    : 0x%x" % fruid
##            else:
##                print "FRU I2C Address                  : 0x%x" % (fruid >> 1)

##            print "FRU Access LUN                   : 0x%x" % ((accessinfo & 0x14) >> 3)
##            print "Private Bus ID                   : 0x%x" % (accessinfo & 0x7)
##            print "Device Type                      : 0x%x" % devtype
##            print "Device Type Modifier             : 0x%x" % devtmod
##            print "FRU Entity ID/Instance           : 0x%x/0x%x" % (fruentid, fruentinst)
##            if fruentid < len(ipmievent.entitytypelist):
##                print "                                 : %s" % ipmievent.entitytypelist[fruentid]
##            print "ID String Type                   : 0x%x" % idtype
##            print "ID String                        : %s" % idstr

##        if rtype == 0x12:
##            try:
##                devaddr, chan, pwrnotify, dcap, rev1, rev2, rev3, entid, entinst, oem, idtype = struct.unpack('<11B', s[5:16])
##                idstr = s[16:]
##            except:
##                print "struct bombed, len = %d" % len(s[5:16])
##                pbytes(s[5:])
##                raise

##            print "Management Controller Address    : 0x%x" % (devaddr >> 1)
##            print "Channel Number                   : 0x%x" % chan
##            print "Power State Notify/Init          : 0x%x" % pwrnotify
##            if pwrnotify & 0x80:
##                print "                                 : ACPI System Power State"
##            if pwrnotify & 0x40:
##                print "                                 : ACPI Device Power State"
##            if pwrnotify & 0x20:
##                print "                                 : Static controller"
##            if pwrnotify & 0x8:
##                print "                                 : Logs Init Agent errors"
##            if pwrnotify & 0x4:
##                print "                                 : Log Init Agent access errors"
##            if not pwrnotify & 0x2 and not pwrnotify & 0x1:
##                print "                                 : Enable event msg generation"
##            if not pwrnotify & 0x2 and pwrnotify & 0x1:
##                print "                                 : Disable event msg generation"
##            if pwrnotify & 0x2 and not pwrnotify & 0x1:
##                print "                                 : Do not init controller"
##            print "Device Capabilities              : 0x%x" % dcap
##            if dcap & 0x80:
##                print "                                 : Chassis Device"
##            if dcap & 0x40:
##                print "                                 : Bridge"
##            if dcap & 0x20:
##                print "                                 : IPMB Event Generator"
##            if dcap & 0x10:
##                print "                                 : IPMB Event Receiver"
##            if dcap & 0x8:
##                print "                                 : FRU Inventory Device"
##            if dcap & 0x4:
##                print "                                 : System Event Log Device"
##            if dcap & 0x2:
##                print "                                 : SDR Repository Device"
##            if dcap & 0x1:
##                print "                                 : Sensor Device"
##            print "Entity ID/Instance               : 0x%x/0x%x" % (entid, entinst)
##            if entid < len(ipmievent.entitytypelist):
##                print "                                 : %s" % ipmievent.entitytypelist[entid]
##            print "ID String                        : %s" % idstr



## MAIN #################################################

def doit():

    global x
    function = None
    arg = None
    # x = None

    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'a:kmlp:r:s')
    except getopt.GetoptError:
        print "Bad option"
        usage()
        sys.exit(1)
        
    for o, a in optlist:
        
        if o == "-a":
            if x == None:
                x = ipmicommon.IPMISession(a)
            else:
                toomanymodes()
        if o == "-k":
            if x == None:
                x = ipmicommon.IPMISystemInterface(interface="KCS")
            else:
                toomanymodes()
        if o == "-m":
            if x == None:
                x = ipmicommon.IPMISystemInterface(interface="SMIC")
            else:
                toomanymodes()
        if o == "-l":
            if function == None:
                function = o
            else:
                toomanyopts()
        if o == "-p":
            if function == None:
                function = o
                arg = a
            else:
                toomanyopts()
        if o == "-r":
            if function == None:
                function = o
                arg = a
        if o == "-s":
            if function == None:
                function = o
            else:
                toomanyopts()
    
    #x = ipmicommon.IPMISession('10.1.19.168')
    #x = ipmicommon.IPMISession('10.0.2.33')
    #x.establish()

    if function == None:
        usage()
        sys.exit(1)

    if x == None:
        usage()
        sys.exit(1)

    x.establish()

    if function == "-p":
        dopower(arg)
    elif function == "-l":
        doseldump()
    elif function == "-r":
        dosensread(arg)
    elif function == "-s":
        dosdrdump()
    else:
        print "eh?"
        usage()
        sys.exit(1)

   

if __name__ == "__main__":
    doit()
    





# ooh!

#           daddr, dlun,saddr,slun,sseq,netfn, cmd, data
# Get Device ID
#ret = x.send(0x20, 0x0, 0x20, 0x0, 0x1, 0x6, 0x1, '')
#pbytes(ret)

# Get Power-on Hours counter
#ret = x.send(0x20, 0x0, 0x20, 0x0, 0x1, 0x0, 0xf, '')
#pbytes(ret)

# Chassis Status
#ret = x.send(0x20, 0x0, 0x20, 0x0, 0x1, 0x0, 0x1, '')
#pbytes(ret)

# Set Auth
#ret = x.send(0x20, 0x0, 0x81, 0x0, 41, 0x6, 0x3b, struct.pack('<B', 4))
#pbytes(ret)

# Get Auth
#ret = x.send(0x20, 0x0, 0x81, 0x0, 41, 0x6, 0x3b, struct.pack('<B', 4))
#pbytes(ret)


# Chassis Control
#ret = x.send(0x20, 0x0, 0x81, 0x0, 41, 0x0, 0x2, struct.pack('<B', int(sys.argv[1])))
#pbytes(ret)

# let's do some damage!
# get capabilities
#ret = x.send(0x20, 0x0, 0x20, 0x0, 0x2, 0x0, 0x0, '')
#pbytes(ret)

# Power Off System
# ew, this requires the chassis' controller address.
#ret = x.send(0x28, 0x0, 0x20, 0x0, 0x2, 0x0, 0x2, struct.pack('<B', 0x0))
#pbytes(ret)
