#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# 
#
# IPMI Event decoding
#


# Take an event's data and return a pair of strings describing it

sensortypelist = ["reserved", "Temperature", "Voltage", "Current", "Fan", "Physical Security", "Platform Security",  "Processor", "Power Supply", "Power Unit", "Cooling Device","Other Unit-based Sensor","Memory","Drive Bay", "POST Memory","Boot Progress","SEL Disabled","Watchdog 1", "System Event","Critical Interrupt","Button","Module/Board","Microcontroller","Add-in Card","Chassis","Chipset","Other FRU","Interconnect","Terminator","System Boot","Boot Error","OS Boot","OS Critical Stop","Slot/Connector","ACPI Power State","Watchdog 2","Platform Alert","Entity Presence","Monitor ASIC","LAN","Management Health", "Battery"]

eventtypelist = ["Unspecified","Threshold","Usage State","State","Predictive Failure", "Limit","Performance","Severity","Device Presence","Device Enable","Device State","Redundancy State","ACPI Power State","Sensor-specific"]

entitytypelist = ['Unspecified', 'Other', 'unknown', 'processor', 'disk or disk bay', 'peripheral bay', 'system management module', 'system board', 'memory module', 'processor module', 'power supply', 'add-in card', 'front panel board', 'back panel board', 'power system board', 'drive backplane', 'system internal expansion board', 'other system board', 'processor board', 'power unit', 'power module', 'power management/distribution board', 'chassis back panel board', 'system chassis', 'sub-chassis', 'other chassis board', 'disk drive bay', 'peripheral bay', 'device bay', 'fan/cooling device', 'cooling unit', 'interconnect', 'memory device', 'system management software', 'BIOS', 'operating system', 'system bus', 'group', 'remote management device', 'external environment', 'battery']

unittypelist = ["unspecified", "Celsius", "Farenheit", 'Kelvin', 'Volt', 'Amp', 'Watt', 'Joule', 'Coulomb', 'VA', 'Nits', 'Lumen', 'Lux', 'Candela', 'kPa', 'psi', 'Newton', 'CFM', 'RPM', 'Hz', 'microsecond', 'millisecond', 'second', 'minute', 'hour', 'day', 'week', 'mil', 'inches', 'feed', 'cubic inches', 'cubic feet', 'millimeter', 'centimeter', 'meter', 'cubic centimeter', 'cubic meter', 'litre', 'fluid ounce', 'radian', 'steradian', 'revolution', 'cycle', 'gravity', 'ounce', 'pound', 'foot-pound', 'ounce-inch', 'gauss', 'gilbert', 'henry', 'millihenry', 'farad', 'microfarad', 'ohm', 'siemen', 'mole', 'becquerel', 'parts per million', 'reserved', 'decibels', 'DbA', 'DbC', 'gray', 'sievert', 'color temperature in Kelvin', 'bit', 'kilobit', 'megabit', 'gigabit', 'byte', 'kilobyte', 'megabyte', 'gigabyte', 'word', 'dword', 'qword', 'line', 'hit', 'miss', 'retry', 'reset', 'overflow', 'underrun', 'collision', 'packet' 'message', 'character', 'error', 'correctable error', 'uncorrectable error']

def decode_event(etype, eoff, stype, data1, data2, data3):

    #print "Decoding event: etype=0x%x eoff=0x%x stype=0x%x" % (etype, eoff, stype)

    etypestr = ""
    especstr = ""
    
    if etype == 0x0:
        etypestr = "Unspecified"
    if etype == 0x1:
        etypestr = "Threshold"
    if etype == 0x2:
        etypestr = "Usage State"
    if etype == 0x3:
        etypestr = "State"
    if etype == 0x4:
        etypestr = "Predictive Failure"
    if etype == 0x5:
        etypestr = "Limit"
    if etype == 0x6:
        etypestr = "Performance"
    if etype == 0x7:
        etypestr = "Severity"
    if etype == 0x8:
        etypestr = "Device Presence"
    if etype == 0x9:
        etypestr = "Device Enable"
    if etype == 0xa:
        etypestr = "Device State"
    if etype == 0xb:
        etypestr = "Redundancy State"
    if etype == 0xc:
        etypestr = "ACPI Power State"
    if etype == 0x6f:
        etypestr = "Sensor-specific"
        if stype == 0x1:
            etypestr = "Temperature"
        if stype == 0x2:
            etypestr = "Voltage"
        if stype == 0x3:
            etypestr = "Current"
        if stype == 0x4:
            etypestr = "Fan"
        if stype == 0x5:
            etypestr = "Physical Security"
            if eoff == 0x0:
                especstr = "General Chassis Intrusion"
            if eoff == 0x1:
                especstr = "Drive Bay Intrusion"
            if eoff == 0x2:
                especstr = "I/O Card Area Intrusion"
            if eoff == 0x3:
                especstr = "Processor Area Intrusion"
            if eoff == 0x4:
                especstr = "LAN Unplugged"
            if eoff == 0x5:
                especstr = "Unauthorized Dock/Undock"
            if eoff == 0x6:
                especstr = "Fan Area Intrusion"
        if stype == 0x6:
            etypestr = "Platform Security"
        if stype == 0x7:
            etypestr = "Processor"
        if stype == 0x8:
            etypestr = "Power Supply"
        if stype == 0x9:
            etypestr = "Power Unit"
            if eoff == 0x0:
                especstr = "Power Down"
            if eoff == 0x1:
                especstr = "Power Cycle"
            if eoff == 0x2:
                especstr = "240VA Power Down"
            if eoff == 0x3:
                especstr = "Interlock Power Down"
            if eoff == 0x4:
                especstr = "AC Lost"
            if eoff == 0x5:
                especstr = "Soft Power Control Failure"
            if eoff == 0x6:
                especstr = "Power Unit Failure"
            if eoff == 0x7:
                especstr = "Predictive Failure"
        if stype == 0xa:
            etypestr = "Cooling Device"
        if stype == 0xb:
            etypestr = "Other Unit-based Sensor"
        if stype == 0xc:
            etypestr = "Memory"
        if stype == 0xd:
            etypestr = "Drive Bay"
        if stype == 0xe:
            etypestr = "POST Memory"
        if stype == 0xf:
            etypestr = "Boot Progress"
            if eoff == 0x0:
                if data2 == 0x0:
                    especstr = "Unspecified"
                if data2 == 0x1:
                    especstr = "No memory installed in system"
                if data2 == 0x2:
                    especstr = "All system memory failed"
                if data2 == 0x3:
                    especstr = "Unrecoverable ATA device failure"
                if data2 == 0x4:
                    especstr = "Unrecoverable system board failure"
                if data2 == 0x5:
                    especstr = "Unrecoverable diskette drive failure"
                if data2 == 0x6:
                    especstr = "Unrecoverable hard disk controller failure"
                if data2 == 0x7:
                    especstr = "Unrecoverable input device failure"
                if data2 == 0x8:
                    especstr = "Removable boot media not found"
                if data2 == 0x9:
                    especstr = "Unrecoverable video board failure"
                if data2 == 0xa:
                    especstr = "No video device detected"
                if data2 == 0xb:
                    especstr = "Firmware ROM corrupted"
                if data2 == 0xc:
                    epsecstr = "CPU voltage mismatch"
                if data2 == 0xd:
                    especstr = "CPU speed mismatch"
            if eoff == 0x1:
                especstr = "System Firmware Hang"
            if eoff == 0x2:
                especstr = "Firmware Progress (not decoded)"
            if eoff == 0x4:
                if data3 == 0x3:
                    especstr = "(HP) Keyboard not connected"
                if data3 == 0x6:
                    especstr = "(HP) Mouse not detected"
        if stype == 0x10:
            etypestr = "SEL Disabled"
        if stype == 0x11:
            etypestr = "Watchdog 1"
        if stype == 0x12:
            etypestr = "System Event"
            if eoff == 0x0:
                especstr = "System Reconfigured"
            if eoff == 0x1:
                # especstr = "OEM System Boot Event"
                # In practice, it's more like this
                especstr = "POST Completed"
            if eoff == 0x2:
                especstr = "Entry Added to Auxiliary Log"
                # data2 has more info
            if eoff == 0x3:
                especstr = "PEF Action"
                # data2 has more info
        if stype == 0x13:
            etypestr = "Critical Interrupt"
        if stype == 0x14:
            etypestr = "Button"
        if stype == 0x15:
            etypestr = "Module/Board"
        if stype == 0x16:
            etypestr = "Microcontroller"
        if stype == 0x17:
            etypestr = "Add-in Card"
        if stype == 0x18:
            etypestr = "Chassis"
        if stype == 0x19:
            etypestr = "Chipset"
        if stype == 0x1a:
            etypestr = "Other FRU"
        if stype == 0x1b:
            etypestr = "Interconnect"
        if stype == 0x1c:
            etypestr = "Terminator"
        if stype == 0x1d:
            etypestr = "System Boot"
        if stype == 0x1e:
            etypestr = "Boot Error"
        if stype == 0x1f:
            etypestr = "OS Boot"
        if stype == 0x20:
            etypestr = "OS Critical Stop"
        if stype == 0x21:
            etypestr = "Slot/Connector"
            if eoff == 0x0:
                especstr = "Fault"
            if eoff == 0x1:
                especstr = "Identify"
            if eoff == 0x2:
                especstr = "Device Installed"
            if eoff == 0x3:
                especstr = "Ready to Install"
            if eoff == 0x4:
                especstr = "Ready to Remove"
            if eoff == 0x5:
                especstr = "Power Off"
            if eoff == 0x6:
                especstr = "Remove Request"
            if eoff == 0x7:
                especstr = "Interlock"
            if eoff == 0x8:
                especstr = "Disabled"
            if data2 == 0x0:
                especstr += " - PCI"
            if data2 == 0x1:
                especstr += " - Drive Array"
            if data2 == 0x2:
                especstr += " - External Peripheral"
            if data2 == 0x3:
                especstr += " - Docking"
            if data2 == 0x4:
                especstr += " - Other Internal"
            if data2 != 0xf:
                especstr += " - Slot %d" % data3
        if stype == 0x22:
            etypestr = "ACPI Power State"
        if stype == 0x23:
            etypestr = "Watchdog 2"
        if stype == 0x24:
            etypestr = "Platform Alert"
        if stype == 0x25:
            etypestr = "Entity Presence"
        if stype == 0x26:
            etypestr = "Monitor ASIC"
        if stype == 0x27:
            etypestr = "LAN"
        if stype == 0x28:
            etypestr = "Management Health"
        if stype == 0x29:
            etypestr = "Battery"
        if stype >= 0xc0 and stype <= 0xff:
            etypestr = "OEM"
        if stype == 0xc7:
            if eoff == 0x0:
                especstr = "(HP) Managment Controller started"
        if stype == 0xc8:
            if eoff == 0x1:
                especstr = "(HP) Remote User %d Logon" % data3
            if eoff == 0x2:
                especstr = "(HP) Remove User %d Logoff" % data3
            if eoff == 0x3:
                especstr = "(HP) Remote User %d Idle Logoff" % data3
        if stype == 0xc9:
            if eoff == 0x3:
                especstr = "(HP) Remote Power Down"
            if eoff == 0x4:
                especstr = "(HP) Remote Power Up"
            if eoff == 0x6:
                especstr = "(HP) Front Panel Power Off"
        if stype == 0xd1:
            if eoff == 0x5:
                especstr = "(HP) Unknown SEL entry"
        
    if etype >= 0x70 and etype <= 0x7f:
                etypestr = "OEM"

    return (etypestr, especstr)
