#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# 
#
# ipmisensor.py
#

import struct

def get_sensor_reading(sess, ownid, olun, sensor):
    ret = sess.send(ownid, olun, 0x81, 0x0, 1, 0x4, 0x2D, struct.pack('<B', int(sensor)))
    # only thresh-type sensors for now
    ccode, reading, evtflag, thresh = struct.unpack('<4B', ret[:4])

    if ccode != 0x0:
        raise ValueError, "Get Sensor Reading error (%d)" % ccode

    return (reading, evtflag, thresh)


def convert_raw(reading, sdr):
    # To print a sensor, we need to fill in this equation:
    #
    # y = Mx + (B * 10^Bexp) * 10^Rexp

    # y = Linearize(y) if the Linearization enum is set.

    # sdr should be the SDR entry for the given reading.

    bexp = int(sdr['b-exponent'])
    rexp = int(sdr['r-exponent'])

    out = (sdr['m'] * reading + (sdr['b'] * (10.0**bexp))) * 10.0**rexp

    #print "Converted reading: %s" % out

    unit = ""

    if sdr['modifier-unit-type'] == 1:
        unit = "%s/%s" % (sdr['base-unit'], sdr['modifier-unit'])
    elif sdr['modifier-unit-type'] == 2:
        unit = "%s * %s" % (sdr['base-unit'], sdr['modifier-unit'])
    else:
        unit = sdr['base-unit']

    return "%s %s" % (out, unit)

          
